import requests
import re
from bs4 import BeautifulSoup


class Scanner(object):
    """
    Finds links for jobs matching certain criteria
    Init - Sets variables, accepts config(a dict)
    Scrape - Executes scraping
    Dump - Caches results to database (or other file based storage)
    """
    def __init__(self, config):
        # The start of the URL (Ex. https://ca.indeed.com)
        self.indeed_base_url = config["indeed_base_url"]
        # The query string containing the advanced search params (Except the start= parameter)
        self.indeed_query_string = config["indeed_query_string"]
        # The page= parameter from the query string
        self.indeed_pagination_string = config["indeed_pagination_string"]
        # The value of start=.
        # The paginate int gets increased by 50 each loop, starting at Page 1 = 0. Page 2 = 50, Page 3 = 100, etc
        self.indeed_pagination_int = 0
        self.job_title_expression = config["job_title_expression"]
        self.top_element = config["top_element"]
        self.atomic_element = config["atomic_element"]
        self.job_title_element = config["job_title_element"]
        self.job_title_descriptor = config["job_title_descriptor"]

        # Data
        self.soup = BeautifulSoup
        self.html_content = ""
        self.results_list = []

    def scrape(self):
        url = self.indeed_base_url + \
              self.indeed_query_string + \
              self.indeed_pagination_string + \
              str(self.indeed_pagination_int)

        self.html_content = requests.get(url).text
        self.soup = BeautifulSoup(self.html_content, "html5lib")
        job_title_pattern = re.compile(self.job_title_expression)
        local_results = []

        for found in self.soup.find_all(self.top_element):
            for atom in found.find_all(self.atomic_element):
                for link in atom.find_all(self.job_title_element, self.job_title_descriptor, href=True):
                    if job_title_pattern.search(link["title"]):
                        link_url = self.indeed_base_url + link["href"]
                        if {"url": link_url, "Title": link["title"]} not in local_results:
                            local_results.append({"url": link_url, "Title": link["title"]})
                        else:
                            pass

        if local_results:
            self.results_list.extend(local_results)

    def dump(self):
        for result in self.results_list:
            print(result)
        return self.results_list

    def exec(self):
        for i in range(0, 20):
            self.scrape()
            self.indeed_pagination_int += 50
        return self.dump()
